function [TokenSUM] = controllerPerform(simResults, constraints)
% function [TokenSUM] = controllerPerform(simResults, constraints)
%     Quantify controller's performance
% Purpose: From the simulation result, this function verifies whether 
%          the controlled system adheres to the constraints and 
%          shows how forcefully the controller performs by comparing 
%          the token values against the constraints. 
%   
%
%  Input arguments: 
%       1) simResults: output of gpensim on controlled sys  
%       2) Constraints: linear inequalities
%
%  Output argument: 
%       1) TokenSUM: sum of tokens in places satisfying constraints
%  Output to screen: Plots showing how the controlled system performs 
%                    against each constraint.
%
%  Functions used:  (none)
%       
%  Reggie.Davidrajuh@uis.no   (c)  10.March.2024

Ps = simResults.No_of_places; % number of places
LOG = simResults.LOG;  % LOG: simulation results

% extract L and B
[L, B] = extractLB(constraints);

% number of constraints
lenConstr = length(constraints);

% number of states in LOG
rows_of_LOG = size(LOG, 1);

% prepare the output file
TokenSUM = zeros(rows_of_LOG, lenConstr);

% the main loop
for ci = 1:lenConstr  % for each constraint
    Li = L(ci, :);  bi = B(ci, :);

    % check each state in LOG
    for rowj = 1:rows_of_LOG
        tokens = LOG(rowj, 1:Ps);  % a marking
        toksum = tokens * Li';     % combination of marking  
        TokenSUM(rowj, ci) = toksum;
        if gt(toksum, bi)  % check whether the state is valid
            disp(['invalid control action in row-',...
                     int2str(rowj)]);
            disp('verification went wrong here ...');
            % load gong.mat; sound(y);
            %return
        end
    end
end
time_series = LOG(:, Ps+6);
TokenSUM = [time_series TokenSUM];

% plot the findings
for ci = 1:lenConstr
    tokens = TokenSUM(:, 1+ci); % extract relevant token sum
    bi = B(ci, :);
    Bs = bi * ones(size(tokens));
    figure, plot(time_series', [Bs tokens], ...
        '-h', 'linewidth', 1, 'MarkerSize', 5);
    ylim([-inf, bi+0.5]), ...
    xlabel('Time'); ylabel('Number of tokens');
    % The title of the plot
    current_constraint = constraints(ci);
    lenCurrConstr = length(current_constraint.L);
    label = ''; 
    for j = lenCurrConstr:-2:2
        label = [label,  '  +  (', ...
            int2str(current_constraint.L{j}), ...
            '*', current_constraint.L{j-1}, ')'];            
    end
    title(['Constraint ', int2str(ci), ':    ',...
        label(4:end), '  <= ', int2str(bi)]);
    legend({'Limit Bi', 'token sum'}); grid on;
end
