function [PN, synthesis_ok] = supervisorSyn(pni, ...
              constraints, uncontrollables, unobservables)
% function [PN, synthesis_ok] = supervisorSyn(pni, ...
%               constraints, uncontrollables, unobservables)
%     supervisor synthesizer
%   
%   Purpose: Synthesis of a supervisor when 
%            some transitions are 
%            uncontrollables and/or unobservables
%
%  Input arguments: 
%       1) Plant: pni - Petri net with initial dynamic
%          (output of in initialdynamics)
%       2) Constraints: set of linear inequalities
%       3) uncontrollables: set of uncontrollable trans
%       4) unobservables: set of unobservable trans
%
%
%  Output argument: 
%       Controlled System as PN 
%  (use 'printsys' to printout the controlled sys)
%  (use 'gpensim' to run simuations on the controlled sys)
%       
%  Functions used:  mro_adm
%                   extractLB, gpensim_2_PNCT,  
%                   imControlledSys, addSlacks2GP,
%                   get_transitions
%
%  NOTE: This functions uses "mro_adm", which is 
%        function in the package "spnbox" and written by 
%        Marian V. Iordache. 
%        Package "spnbox" accompany the book: 
%        Iordache and Antsaklis (2007) n
%
%  Reggie.Davidrajuh@uis.no   (c)  08.March.2024

global PN
PN = pni;

% extract L and B from constraints
[L, B] = extractLB(constraints);  

% incidence matrix and mu0 of plant
[~,~,Dp] = gpensim_2_PNCT(PN.incidence_matrix); 
mu0 = transpose(PN.initial_marking);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some transitions are uncontrollable / unobservable 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nuc = 0; uc_ts = []; % initial assumption on uncontrollables
nuo = 0; uo_ts = []; % initial assumption on unobservables 

if ge(nargin, 3)  % get the set of uncontrollable transitions
    [nuc, uc_ts] = get_transitions(PN, uncontrollables);
end
if ge(nargin, 4)  % get the set of unobservable transitions
    [nuo, uo_ts] = get_transitions(PN, unobservables);
end

if or(nuc, nuo) % there are uncontrollable/unobservable trans   
    [L1, B1, R1, R2, how, dhow] = mro_adm(L, B, Dp, uc_ts, uo_ts, mu0, 0);
else
    % simple supervisor, with all trans observable & controllable
    L1 = L; B1 = B; how = 'ok';
end

synthesis_ok = strcmp(how, 'ok');
if not(synthesis_ok)
    %load gong.mat; sound(y);
    disp('******************************************');   
    disp('******  Controller not feasible !! ******');   
    disp('******************************************');   
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% controller
Dc = -1 * L1 * Dp;  
muC = B1  - L1 * mu0; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%% Update Controlled Sys %%%%%%%%%%%%%%%%%%%%%%%%%
% update incidence matrix for controlled system
imControlledSys(Dc, size(L,1));
% add slack variables to the list of global_places 
addSlacks2GP(muC);
