function [ISCN] = is_causalNet()
% function [ISCN] = is_causalNet()
%       Is this a Causal Net?
%
% Purpose:
% This function checks whether the Petri net is a Causal Net. 
% 
% Input arguments:
%
% Output arguments:  BOOLEAN value (true of false) 
% 
%
% Used by: ()
%
%       Reggie.Davidrajuh@uis.no (c) January 2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global PN

Ps = PN.No_of_places;  
Ts = PN.No_of_transitions;
A = PN.incidence_matrix;

% incidence matrices
Ainn = A(:, 1:Ps);
Aout = A(:, Ps+1:2*Ps);

ISN = ones(1, 4); % initial assumption: a Casual Net 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check-1: all places have MAX one output transition

% column sum of Ainn gives number of output transitions 
columsum_Ainn = sum(Ainn);
Places = find(columsum_Ainn > 1);
if not(isempty(Places))
    dispText = '-> Check-1: Places with more than one output transition: ';
    for i = 1: numel(Places)
        dispText = [dispText, '  ', pname(Places(i))]; 
    end
    disp(dispText);
    ISN(1) = 0; 
else
    disp('-> Check-1: Places with more than one output transition: (none)');
end
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check-2: all places have MAX one input transition

% column sum of Ainn gives number of input transitions 
columsum_Aout = sum(Aout);
Places = find(columsum_Aout > 1);
if not(isempty(Places))
    dispText = '-> Check-2: Places with more than one input transition: ';
    for i = 1: numel(Places)
        dispText = [dispText, '  ', pname(Places(i))]; 
    end
    disp(dispText);
    ISN(2) = 0; 
else
    disp('-> Check-2: Places with more than one input transition: (none)');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check-3: No cold transitions 
% (cold transitions: transition with zero input places)

rowsum_Ainn = sum(Ainn,2);
Trans = find(rowsum_Ainn == 0);
if not(isempty(Trans))
    dispText = '-> Check-3: Cold transitions:';
    for i = 1: numel(Trans)
        dispText = [dispText, '  ',...
                PN.global_transitions(Trans(i)).name];
    end
    disp(dispText);
    ISN(3) = 0; 
else
    disp('-> Check-3: Cold transitions: (none)');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check-4: No cycles
% (if there are cycles, number of strongly connected 
%   components will be smaller than the total elements

SCC = stronglyconn(PN, 1); % find the strongly connected components
% number of ROWS is number strongly connected components
number_of_SCC  = size(SCC,1); 
if lt(number_of_SCC, Ps + Ts)
    disp('-> Check-4: There are cycles in the Petri Net');
    ISN(4) = 0; 
else
    disp('-> Check-4: There are NO cycles in the Petri Net');
end
    
ISCN = prod(ISN,2);

