function [pntype] = pntype()
% function [classtype] = pntype(png)
%       Petri Net types 
% Purpose:
% This function checks the type and class of Petri net. 
% This function returns a vector of flags representing 
% the following information (output variable classtype):
%   flag-1: Binary PN
%   flag-2: PN State Machine
%   flag-3: Marked (Event) Graph
%   flag-4: Timed Petri Net
%   flag-5: Number of Strongly Connected components
%   flag-6: Casual net
% 
% Uses:
%   pnt = pntype(PN)   
% 
%
% Used by: MSF
%
%       Reggie.Davidrajuh@uis.no (c) February 2022
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global PN

Ps = PN.No_of_places;  
Ts = PN.No_of_transitions;
A = PN.incidence_matrix;

pntype = zeros(1, 8); % prepare to store type info

disp([' **** ', PN.name, ' ****']);

%%%%%%%%%% Flag-1: Binary PN (all the arc weights are delfault '1')
pntype(1) = is_binaryPN();% set flag for Binary PN
if pntype(1)
    disp('This is a Binary Petri Nets.');
end

%%%%%%%%%% Flag-2: State Machine (every transition has exactly 
%           one input and output place
pntype(2) = is_stateMachine(); % set flag for Petri Net State Machine
if pntype(2) 
    disp('This is a Petri Net State Machine.');
end

%%%%%%%%%% Flag-3: Event (Marked) Graph (every place has exactly 
%          one input and one output transition
pntype(3) = is_markedGraph(); % set flag for Petri Net Marked Graph
if pntype(3) 
    disp('This is an Event Graph (Marked Graph).');
end


%%%%%%%%%% Flag-4: Timed/untimed Petri net
% Timed petri net: at least one firing time is non-zero
pntype(4) = is_timedPN; % Timed or untimed Petri net
if pntype(4)
    disp('This is a Timed Petri net.');
else
    disp('This is an untimed Petri net.');
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Flag-5: strongly connectedness 
SCC = stronglyconn(PN, 1); % find the strongly connected components
% number of ROWS is number strongly connected components
pntype(5) = size(SCC,1); 
disp(['Number of strongly connected components: ', int2str(pntype(5))]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Flag-6: Causal Net
pntype(6) = is_causalNet; % Causal Net?
if pntype(6)
    disp('This is a causal net.');
else
    disp('This is not causal net.');
end
